﻿using Microsoft.AspNetCore.Mvc;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using TestLibrary;

namespace TestServer.Controllers
{
    /// <summary>
    /// 테스트 컨트롤러
    /// </summary>
    [Route("api/[controller]")]
    public class TestController : Controller
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 샘플 데이터
        /// </summary>
        private static string _sampleData;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - TestController()

        /// <summary>
        /// 생성자
        /// </summary>
        static TestController()
        {
            StringBuilder stringBuilder = new StringBuilder();

            for(int i = 0; i < 100; i++)
            {
                stringBuilder.Append("ABCDEFGHIJABCDEFGHIJ");
                stringBuilder.Append("ABCDEFGHIJABCDEFGHIJ");
                stringBuilder.Append("ABCDEFGHIJABCDEFGHIJ");
                stringBuilder.Append("ABCDEFGHIJABCDEFGHIJ");
                stringBuilder.Append("ABCDEFGHIJABCDEFGHIJ");
            }

            _sampleData = stringBuilder.ToString();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region GET 데이터 구하기 - GetData(tag)

        /// <summary>
        /// GET 데이터 구하기
        /// </summary>
        /// <param name="tag">태그</param>
        /// <returns>값 열거 가능형</returns>
        [HttpGet]
        [Route("GetData")]
        public IEnumerable<ValueModel> GetData(string tag)
        {
            foreach(int i in Enumerable.Range(1, 1000000))
            {
                yield return new ValueModel()
                {
                    ID   = i,
                    Data = _sampleData,
                    Tag  = tag
                };
            }
        }

        #endregion
        #region POST 데이터 구하기 - PostData(source)

        /// <summary>
        /// POST 데이터 구하기
        /// </summary>
        /// <param name="source">소스 값</param>
        /// <returns>값 열거 가능형</returns>
        [HttpPost]
        [Route("PostData")]
        public IEnumerable<ValueModel> PostData([FromBody] ValueModel source)
        {
            foreach(int i in Enumerable.Range(1, 1000000))
            {
                yield return new ValueModel()
                {
                    ID   = i,
                    Data = _sampleData,
                    Tag  = source.Tag
                };
            }
        }

        #endregion
    }
}